#!/usr/bin/env python2
# -*- coding: utf-8 -*-

# SAT plugin to detect language (experimental)
# Copyright (C) 2009-2019 Jérôme Poisson (goffi@goffi.org)

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from sat.core.i18n import _
from sat.core.constants import Const as C
from sat.core.log import getLogger
from twisted.internet import defer
from twisted.words.protocols.jabber import jid
from wokkel import disco, iwokkel
from zope.interface import implements
from twisted.words.protocols.jabber.xmlstream import XMPPHandler

log = getLogger(__name__)


PLUGIN_INFO = {
    C.PI_NAME: "File Sharing Invitation",
    C.PI_IMPORT_NAME: "FILE_SHARING_INVITATION",
    C.PI_TYPE: "EXP",
    C.PI_PROTOCOLS: [],
    C.PI_DEPENDENCIES: ["XEP-0329"],
    C.PI_RECOMMENDATIONS: [],
    C.PI_MAIN: "FileSharingInvitation",
    C.PI_HANDLER: "yes",
    C.PI_DESCRIPTION: _(u"Experimental handling of invitations for file sharing"),
}

NS_FILE_SHARING_INVITATION = "https://salut-a-toi/protocol/file-sharing-invitation:0"
INVITATION = '/message/invitation[@xmlns="{ns_invit}"]'.format(
    ns_invit=NS_FILE_SHARING_INVITATION
)


class FileSharingInvitation(object):

    def __init__(self, host):
        log.info(_(u"File Sharing Invitation plugin initialization"))
        self.host = host

    def getHandler(self, client):
        return FileSharingInvitationHandler(self)

    def sendFileSharingInvitation(
            self, client, invitee_jid, service, repos_type=None, namespace=None,
            path=None):
        """Send an invitation in a <message> stanza

        @param invitee_jid(jid.JID): entitee to send invitation to
        @param service(jid.JID): file sharing service
        @param repos_type(unicode, None): type of files repository, can be:
            - None, "files": files sharing
            - "photos": photos album
        @param namespace(unicode, None): namespace of the shared repository
        @param path(unicode): pubsub id
        """
        mess_data = {
            "from": client.jid,
            "to": invitee_jid,
            "uid": "",
            "message": {},
            "type": C.MESS_TYPE_CHAT,
            "subject": {},
            "extra": {},
        }
        client.generateMessageXML(mess_data)
        event_elt = mess_data["xml"].addElement("invitation", NS_FILE_SHARING_INVITATION)
        event_elt[u"service"] = service.full()
        if repos_type is not None:
            event_elt[u"type"] = repos_type
        if namespace is not None:
            event_elt[u"namespace"] = namespace
        if path is not None:
            event_elt[u"path"] = path
        client.send(mess_data[u"xml"])

    @defer.inlineCallbacks
    def onInvitation(self, message_elt, client):
        invitation_elt = message_elt.invitation
        try:
            service = jid.JID(invitation_elt["service"])
        except (RuntimeError, KeyError):
            log.warning(_(u"Bad invitation, ignoring: {xml}").format(
                xml=message_elt.toXml()))
            return

        repos_type = invitation_elt.getAttribute(u"type", u"files")
        namespace = invitation_elt.getAttribute(u"namespace")
        path = invitation_elt.getAttribute(u"path")
        if repos_type == u"files":
            type_human = _(u"file sharing")
        elif repos_type == u"photos":
            type_human = _(u"photos album")
        log.info(_(
            u'{profile} has received an invitation for a files repository ({type_human}) '
            u'with namespace "{namespace}" at path [{path}]').format(
            profile=client.profile, type_human=type_human, namespace=namespace, path=path)
            )


class FileSharingInvitationHandler(XMPPHandler):
    implements(iwokkel.IDisco)

    def __init__(self, plugin_parent):
        self.plugin_parent = plugin_parent

    def connectionInitialized(self):
        self.xmlstream.addObserver(
            INVITATION, self.plugin_parent.onInvitation, client=self.parent
        )

    def getDiscoInfo(self, requestor, target, nodeIdentifier=""):
        return [
            disco.DiscoFeature(NS_FILE_SHARING_INVITATION),
        ]

    def getDiscoItems(self, requestor, target, nodeIdentifier=""):
        return []
